import { UANodeSet } from "./UANodeSet";
import { UABaseNode } from "./UABaseNode";
import { NamespaceTable } from "./NameSpaceTable";
import JSZip from "jszip";

export class AddressSpace{

    nodeMap:  Map<string, UABaseNode>;

    constructor(public nodesets: UANodeSet[]) {
        this.reIndex(nodesets);
        this.nodeMap=this.buildNodeMap(nodesets);
        this.resolveChildren(nodesets, this.nodeMap);
    }

    public findNode(nodeId: string):UABaseNode|undefined {
        return this.nodeMap.get(nodeId)
    }


    private reIndex(nodesets:UANodeSet[]) {
        const nst=new NamespaceTable();
        for(const nodeset of nodesets) {
            nodeset.reIndex(nst);
        }
    }
    
    private buildNodeMap(nodesets:UANodeSet[]) {
        const nm=new Map<string, UABaseNode>();
        for(const nodeset of nodesets) {
            for(const node of nodeset.nodes) {
                nm.set(node.nodeId.toString(), node);
            }
        }
        return nm;
    }

    private resolveChildren(nodesets:UANodeSet[], nm:Map<string, UABaseNode>) {
        for(const nodeset of nodesets) {
            nodeset.resolveChildren(nm);
        }
    }


    static async load(files: string[]) {
        const promises:Promise<UANodeSet>[] = []
        for(const file of files) {
            promises.push(UANodeSet.load(file));
        }
        const sets:UANodeSet[]= [];
        for(const p of promises) {
            sets.push(await p)
        }
        return new AddressSpace(sets);
    }

    public export() {
        const zip = new JSZip();
        const fileNames:string[]=[];
        for(const ns of this.nodesets) {
            fileNames.push(ns.fileName);
            zip.file(ns.fileName, ns.toXML().toString());
        }
        zip.file("project.json", JSON.stringify(fileNames));
        return zip.generateAsync({type:'blob'});
    }
}