import type { NamespaceTable } from "./NameSpaceTable";
import { NodeId } from "./NodeId";
import { ReferencyType, UAReference } from "./UAReference";
import { assert } from "@/util/assert";
import { XMLElem, type IToXML } from "@/util/XmlElem";

export class UABaseNode implements IToXML{
    children: UABaseNode[]=[];
    constructor(public nodeId: NodeId,
                public browseName: string,
                public displayName: string,
                public references: UAReference[]) {
    }

    reIndex(nst: NamespaceTable, onst: NamespaceTable) {
        const nsName=onst.getUri(this.nodeId.namespace);
        assert(nsName!=undefined)
        const newIndex=nst.getIndex(nsName);
        assert(newIndex!=undefined)
        this.nodeId.namespace=newIndex;

        for(const uaref of this.references) {
            uaref.reIndex(nst, onst);
        }
    }

    resolveChildren(nm: Map<string, UABaseNode>) {
        for(const ref of this.references) {
            const node=nm.get(ref.ref.toString()) 
            if(node==undefined) {
                //TODO: parse all types/nodes
                continue;
            }
            switch(ref.referenceType) {
                case ReferencyType.HasComponent:
                case ReferencyType.Organizes:
                case ReferencyType.HasProperty:
                case ReferencyType.HasSubtype:
                    if(ref.isForward) {
                        if(!this.children.includes(node))
                            this.children.push(node);
                    } else {
                        if(!node.children.includes(this))
                            node.children.push(this);
                    }
                    break;
                }
        }
    }

    static  fromXML(xmlObject: any): UABaseNode{
        const xmlReferences=xmlObject['References'];
        const references:UAReference[]=[];
        for(const xmlref of xmlReferences.Reference) {
            references.push(UAReference.fromXML(xmlref));
        }
        const ua=new UABaseNode(NodeId.coerceNodeId(xmlObject['@_NodeId']), xmlObject['@_BrowseName'], xmlObject['DisplayName'], references);
        return ua;
    }

    toXML(): XMLElem {
        throw new Error("UABaseNode has no xml rep; implement in subtype.");
    }
}