export class XMLElem {
    name: string;
    value: string;
    elements: XMLElem[]=[];
    attributes: XmlAttr[]=[];

    constructor(name: string, value?:string) {
        this.name=name;
        this.value=value||'';
    }

    public add(elem: XMLElem): XMLElem {
        this.elements.push(elem);
        return elem;
    }

    public attr(name:string, value: string|boolean): XMLElem {
        this.attributes.push(new XmlAttr(name, value?.toString()));
        return this;
    }

    public toString(level: number=0): string {
        let s=" ".repeat(level+1) + `<${this.name} `;
        for(const attr of this.attributes) 
            s+=`${attr.name}="${attr.value}" `;
        s=s.slice(0, -1);
        s+=`>`
        for(const elem of this.elements) {
            s+="\n"+" ".repeat(level) + elem.toString(level+1);
        }
        if(this.value && this.elements.length==0) {
            s+=this.value;
        }
        if(this.elements.length>0)
            s+=" ".repeat(level);
        s+=`</${this.name}>\n`
        return s;
    }
}

class XmlAttr {
    constructor(public name: string, 
                public value: string) {
                }

}

export interface IToXML {
    toXML() :XMLElem;
}